# Build & Run

Follow this guide to build the modules needed to run the apps in the local environment using `npm`.

* [Build](#build)
    * [Building using npm](#building-using-npm)
* [Run](#run)
* [Run Asgardeo Console pointing to Dev Cloud Backend](#run-asgardeo-console-pointing-to-dev-cloud-backend)
    * [M1: Run Asgardeo Console using asgardeo app-extensions repo](#m1-run-asgardeo-console-using-asgardeo-app-extensions-repo)
    * [M2: Run Asgardeo Console using identity-apps repo](#m2-run-asgardeo-console-using-identity-apps-repo)

Before building, run the following command from the root of the `asgardeo-app-extensions` (where the root `pom.xml` is located).

```shell
mvn antrun:run@build-pack
```

Run the following command in the above same root directory.

```shell
mvn antrun:run@init-extensions
```

## Build

### Building using npm

Then navigate to the `target/identity-apps/apps/console/scripts/` and execute the `pre-install.js`.

```shell
node pre-install.js
```
After that, navigate back to the `target/identity-apps/` and execute the build command.

```shell
npm run build
```

## Run

To run the apps, navigate to the specific folder and execute following command.

```shell
# To start console on port 443
cd apps/console
npm start
```
(If running on dev environment, follow the file configurations mentioned in [asgardeo-app-extensions](https://github.com/wso2-enterprise/asgardeo-app-extensions/blob/master/README.md) after `mvn clean install`)

## Run Asgardeo Console pointing to Dev Cloud Backend

Look through here if you want to run Asgardeo Console pointing to Dev Cloud Backend. This will help you to work on UI changes.


## M1 Run Asgardeo Console using asgardeo app-extensions repo

**Pre-requisites**

```
- Create a fork of the asgardeo app-extensions repo.
- Clone the repo and set upstream
```
> Refer `Setting Up Development Environment` to set up repos.

> Link to [asgardeo app-extensions](https://github.com/wso2-enterprise/asgardeo-app-extensions) repo.

1. Execute `mvn clean install` from the root of the repo.

✍️  Note: Success of the command will generate a target folder that consists of the build.

2. Update `index.html` and `deployment.config.json` files in the following locations.

```
asgardeo-app-extensions/target/identity-apps/apps/console/src/index.html

asgardeo-app-extensions/target/identity-apps/apps/console/src/public/deployment.config.json
```

✍️  Note: Refer [Change index and deployment files](#change-index-and-deployment-files) to update the files with Asgardeo configurations.

3. Open a new terminal and execute the following commands from `target/identity-apps/apps`.

- Run the pre-install script to install dependencies (ex: microsoft).
```
node console/scripts/pre-install.js
```
- Run the command to build the lerna modules (theme, i18n and etc.).
```
npx lerna bootstrap
```
- Build the theme specific Dev components.
```
npx lerna run build:dev --scope @wso2is/theme
```

4. Open a new terminal in `target/identity-apps/apps/console` and execute the following command to run the console.

```
sudo npm start -- env DEV_SERVER_PORT=443 env DISABLE_DEV_SERVER_HOST_CHECK=true
```

✨  Note: Running the console will take sometime and check whether the terminal shows `No issues found` in the end of logs.

5. Add the following host mapping to map localhost to development cloud's console host.
```
127.0.0.1 dev.console.asgardeo.io
```

💡 Note: In macOS, usually host file is located at `/etc/hosts`.

6. Bypass the Certificate warning.

💡 Note: If you get an invalid certificate warning and don't see a button to bypass, try typing `thisisunsafe` at the window (Only works in Chrome).

7. You can make your changes inside the `target/identity-apps/apps/console` folder and check whether they are reflected in the browser (URL: `dev.console.asgardeo.io`)

## M2 Run Asgardeo Console using identity-apps repo

**Pre-requisites**

```
- Create a fork of the identity-apps and asgardeo app-extensions repo.
- Clone the repos and set upstreams
```
> Refer `Setting Up Development Environment` to set up repos.

Link to [asgardeo app-extensions](https://github.com/wso2-enterprise/asgardeo-app-extensions) repo.
Link to [identity-apps](https://github.com/wso2/identity-apps) repo.

1. Execute `mvn clean install` from the root of the repo.

2. Get asgardeo specific components
Explaination:
Copy extensions folder in `asgardeo-app-extensions/extensions/console/src/extensions`.
Paste to `identity-apps/apps/console/src/`
Make sure you OVERWRITE when pasting.

✨  Tip: You can use intellij IDE to have the overwrite option when pasting

3. Get the asgardeo theme

Get a copy of `identity-apps/modules/theme/src/themes/default`
Rename the copied folder as `asgardio`
Copy asgardeo-app-extensions/extensions/theme-extensions/asgardio
Paste to `identity-apps/modules/theme/src/themes/`
Make sure you OVERWRITE when pasting.

4. Update `index.html` and `deployment.config.json` files in the following locations.

```
asgardeo-app-extensions/target/identity-apps/apps/console/src/index.html

asgardeo-app-extensions/target/identity-apps/apps/console/src/public/deployment.config.json
```

✍️  Note: Refer [Change index and deployment files](#change-index-and-deployment-files) to update the files with Asgardeo configurations.

5. Open a new terminal and execute the following commands from the root of identity-apps

- Build the theme specific Dev components.
```
npx lerna run build:dev --scope @wso2is/theme
```

- Run the command to build the lerna modules (theme, i18n and etc.).
```
npx lerna bootstrap
```

- Build the modules
```
npm run build:modules
```

- Run the pre-install script to install dependencies (ex: microsoft).

Copy the pre-install script in `app-extensions/script-extensions/pre-install.js` and paste it to `identity-apps/apps/console/scripts/`
```
node console/scripts/pre-install.js
```

6. Open a new terminal in `identity-apps/apps/console` and execute the following command to run the console.
```
sudo npm start -- env DEV_SERVER_PORT=443 env DISABLE_DEV_SERVER_HOST_CHECK=true from identity apps/console
```

✨  Note: Running the console will take sometime and check whether the terminal shows `No issues found` in the end of logs.

7. Add the following host mapping to map localhost to development cloud's console host.
```
127.0.0.1 dev.console.asgardeo.io
```

💡 Note: In macOS, usually host file is located at `/etc/hosts`.

8. Bypass the Certificate warning.

💡 Note: If you get an invalid certificate warning and don't see a button to bypass, try typing `thisisunsafe` at the window (Only works in Chrome).

9. You can make your changes in the `identity-apps/console` and check whether they are reflected in the browser (URL: `dev.console.asgardeo.io`)

## Change index and deployment files

### Change the `index.html` file

##### Step 1
- Open up the `index.html` file located at the root of `src` folder. And change the **contextPathGlobal** from `/console` to `""`.

```html
    <script>
        var contextPathGlobal = "";
        ...
    </script>
```

##### Step 2

Do the following path changes in the `index.html`.

###### Theme

```html
    <!-- Remove <%= htmlWebpackPlugin.options.publicPath %> & change `default` -> `asgardio` -->
    <link href="/libs/themes/asgardio/theme.<%= htmlWebpackPlugin.options.themeHash %>.min.css" rel="stylesheet" type="text/css"/>

```

###### Favicon

```html
    <!-- Remove <%= htmlWebpackPlugin.options.publicPath %> & change `default` -> `asgardio` -->
    <link rel="shortcut icon" href="/libs/themes/asgardio/assets/images/branding/favicon.ico" />
```
### Change the `deployment.config.json` file

##### Step 1

Change the app base name from `"console"` to `""`.

```json
{
    "appBaseName": "",
}
```

##### Step 2

Change the `Server Origin`.

URL added will depend on the environment you wish to point. In this case dev cloud. (https://dev.api.asgardeo.io)

```json
{
    "idpConfigs": {
        "serverOrigin": "https://dev.api.asgardeo.io",
    },
    "serverOrigin": "https://dev.api.asgardeo.io",
}
```

##### Step 3

**Update IDP resource endpoints: -**

Update the `authorizeEndpointURL` & `logoutEndpointURL` accordingly.

> :bulb: Skipping this step will result in unexpected session persistence issue.

**Console**

```json
{
    "idpConfigs": {
        "authorizeEndpointURL": "${serverOrigin}/${tenantPrefix}/${superTenantDomain}/oauth2/authorize?ut=${userTenantDomain}",
        "logoutEndpointURL": "${serverOrigin}/${tenantPrefix}/${superTenantDomain}/oidc/logout",
        "oidcSessionIFrameEndpointURL": "${serverOrigin}/${tenantPrefix}/${superTenantDomain}/oidc/checksession"
    }
}
```

**My Account**

```json
{
    "idpConfigs": {
        "authorizeEndpointURL": "${serverOrigin}/${tenantPrefix}/${userTenantDomain}/common/oauth2/authorize",
        "logoutEndpointURL": "${serverOrigin}/${tenantPrefix}/${userTenantDomain}/common/oidc/logout",
        "oidcSessionIFrameEndpointURL": "${serverOrigin}/${tenantPrefix}/${userTenantDomain}/common/oidc/checksession"
    }
}
```

**Also, update the following paths values: - (Applies for both `myacccount` and `console`)**

```txt
    "loginCallbackPath": "/login",
    "logoutCallbackPath": "/logout",
    "routePaths": {
        "login": "/login",
        "logout": "/logout"
    }
```

##### Step 4

Add super tenant proxy.

> :bulb: With the tenant wise cookie handling, super tenant login flows have tenant path as /t/carbon.super. `superTenantProxy` is used to mask `carbon.super`.

```json
{
    "superTenantProxy" : "a",
}
```

> :bulb: Read more about these configs [here](https://github.com/wso2/identity-apps/blob/master/docs/CONFIGURATION.md).

##### Step 5

Add extension configs.

**Console**

```json
{
    "extensions": {
        "applicationInsightsEnabled": false,
        "applicationInsightsCookieDomain": "",
        "applicationInsightsCookiePostfix": "",
        "applicationInsightsInstrumentationKey": "",
        "applicationInsightsProxyEndpoint": "",
        "collaboratorUsernameRegex": "^[\\u00C0-\\u00FFa-zA-Z0-9.\\-_]+@[a-zA-Z0-9.\\-]+\\.[a-zA-Z]{2,10}$",
        "feedbackEndPoint": "https://dev.portal.asgardeo.io/api/cloud/v1/feedback/"
    }
}
```

**My Account**

```json
{
    "extensions": {
        "applicationInsightsEnabled": false,
        "applicationInsightsCookieDomain": "",
        "applicationInsightsCookiePostfix": "",
        "applicationInsightsInstrumentationKey": "",
        "applicationInsightsProxyEndpoint": ""
    }
}
```
